{ stdenv
, lib
, fetchFromGitHub
, fetchpatch
, bison
, pam

, withTimestamp ? true
}:

stdenv.mkDerivation rec {
  pname = "doas";
  version = "6.6.1";

  src = fetchFromGitHub {
    owner = "Duncaen";
    repo = "OpenDoas";
    rev = "v${version}";
    sha256 = "07kkc5729p654jrgfsc8zyhiwicgmq38yacmwfvay2b3gmy728zn";
  };

  # otherwise confuses ./configure
  dontDisableStatic = true;

  configureFlags = [
    (lib.optionalString withTimestamp "--with-timestamp") # to allow the "persist" setting
    "--pamdir=${placeholder "out"}/etc/pam.d"
  ];

  patches = [
    # Allow doas to discover binaries in /run/current-system/sw/{s,}bin and
    # /run/wrappers/bin
    ./0001-add-NixOS-specific-dirs-to-safe-PATH.patch

    (fetchpatch {
      name = "CVE-2019-25016.patch";
      url = "https://gitlab.alpinelinux.org/alpine/aports/-/raw/9e259950190c924b4a17825aad2d7cee87fbd75b/main/doas/reset-path.patch";
      sha256 = "sha256-Nx+i2+HOtCQ65AC6aJGAq12Pxr2BLDrKlt1PTUstDug=";
    })
  ];

  postPatch = ''
    sed -i '/\(chown\|chmod\)/d' bsd.prog.mk
  '';

  buildInputs = [ bison pam ];

  meta = with lib; {
    description = "Executes the given command as another user";
    homepage = "https://github.com/Duncaen/OpenDoas";
    license = licenses.isc;
    platforms = platforms.linux;
    maintainers = with maintainers; [ cole-h cstrahan ];
  };
}
